#include "common.h"

void spi_transfer(uint8_t* tx, uint8_t* rx);

static void interrupt_handler()
{
	uint32_t status = MAP_SPIIntStatus(GSPI_BASE,true);
	MAP_SPIIntClear(GSPI_BASE,SPI_INT_EOW);

	spi_transfer(tx_buffer,rx_buffer);

	if(!(status &  SPI_INT_EOW))
	{
		Message("Error: Unexpected SPI interrupt!\n\r");
		while(1){};
	}
}

/*
* Here we should get rid of everything, but the SetupTransfer calls
*/
void spi_transfer(uint8_t* tx, uint8_t* rx)
{
	MAP_SPIReset(GSPI_BASE);

	//UDMAInit();

    MAP_SPIConfigSetExpClk(GSPI_BASE,MAP_PRCMPeripheralClockGet(PRCM_GSPI),
                     SPI_IF_BIT_RATE,SPI_MODE_SLAVE,SPI_SUB_MODE_0,
                     (SPI_HW_CTRL_CS |
                     SPI_4PIN_MODE |
                     SPI_TURBO_OFF |
                     SPI_CS_ACTIVEHIGH |
                     SPI_WL_8));	
	MAP_SPIIntRegister(GSPI_BASE,interrupt_handler);

	SetupTransfer(UDMA_CH30_GSPI_RX,UDMA_MODE_BASIC,DMA_SIZE,
                UDMA_SIZE_8,UDMA_ARB_1,
                (void *)(GSPI_BASE + MCSPI_O_RX0),UDMA_SRC_INC_NONE,
                rx,UDMA_DST_INC_8);

	SetupTransfer(UDMA_CH31_GSPI_TX,UDMA_MODE_BASIC,DMA_SIZE,
                UDMA_SIZE_8,UDMA_ARB_1,
                tx,UDMA_SRC_INC_8,
				(void *)(GSPI_BASE + MCSPI_O_TX0),UDMA_DST_INC_NONE);	
	MAP_SPIWordCountSet( GSPI_BASE, DMA_SIZE);
	MAP_SPIFIFOLevelSet( GSPI_BASE, 1, 1);
	MAP_SPIFIFOEnable( GSPI_BASE, SPI_RX_FIFO);
	MAP_SPIFIFOEnable( GSPI_BASE, SPI_TX_FIFO); 
	MAP_SPIDmaEnable(GSPI_BASE,SPI_RX_DMA|SPI_TX_DMA);
	MAP_SPIIntEnable( GSPI_BASE, SPI_INT_EOW);
	MAP_SPIEnable(GSPI_BASE);

}
void spi_setup()
{
	MAP_SPIReset(GSPI_BASE);

	UDMAInit();

    MAP_SPIConfigSetExpClk(GSPI_BASE,MAP_PRCMPeripheralClockGet(PRCM_GSPI),
                     SPI_IF_BIT_RATE,SPI_MODE_SLAVE,SPI_SUB_MODE_0,
                     (SPI_HW_CTRL_CS |
                     SPI_4PIN_MODE |
                     SPI_TURBO_OFF |
                     SPI_CS_ACTIVEHIGH |
                     SPI_WL_8));	
	MAP_SPIIntRegister(GSPI_BASE,interrupt_handler);

	MAP_SPIWordCountSet( GSPI_BASE, DMA_SIZE);
	MAP_SPIFIFOLevelSet( GSPI_BASE, 1, 1);
	MAP_SPIFIFOEnable( GSPI_BASE, SPI_RX_FIFO);
    MAP_SPIFIFOEnable( GSPI_BASE, SPI_TX_FIFO); 
	MAP_SPIDmaEnable(GSPI_BASE,SPI_RX_DMA|SPI_TX_DMA);
	MAP_SPIIntEnable( GSPI_BASE, SPI_INT_EOW);
	MAP_SPIEnable(GSPI_BASE);
}

int main()
{

	init();
	// initialize buffers with some known value
	memset(tx_buffer,0x55,sizeof(tx_buffer));
	memset(rx_buffer,0x22,sizeof(rx_buffer));
	
	spi_setup();
	spi_transfer(tx_buffer,rx_buffer);

    Message("Enabled SPI Interface in Slave Mode!\n\r");

    while(1)
    {
		memcpy(tx_buffer,rx_buffer,DMA_SIZE);
		// here we could also change the tx_buffer
		// e.g.	tx_buffer[0] = 18;
    }
	return 0;
}

